@@ -3,7 +3,7 @@ package Mojolicious::Plugin::DataTables::SSP::Results;
 use Mojo::Base -base;
 use Mojo::JSON qw(encode_json);
 
-our $VERSION = '1.03';
+our $VERSION = '2.00';
 
 has 'draw';
 has 'records_total';
@@ -72,4 +72,39 @@ L<Mojolicious::Plugin::DataTables::SSP::Results> implements the following method
 
 L<Mojolicious>, L<Mojolicious::Guides>, L<https://mojolicious.org>, L<Mojolicious::Plugin::DataTables>, L<Mojolicious::Plugin::DataTables::SSP::Params>.
 
+
+=head1 SUPPORT
+
+=head2 Bugs / Feature Requests
+
+Please report any bugs or feature requests through the issue tracker
+at L<https://github.com/giterlizzi/perl-Mojolicious-Plugin-DataTables/issues>.
+You will be notified automatically of any progress on your issue.
+
+=head2 Source Code
+
+This is open source software.  The code repository is available for
+public review and contribution under the terms of the license.
+
+L<https://github.com/giterlizzi/perl-Mojolicious-Plugin-DataTables>
+
+    git clone https://github.com/giterlizzi/perl-Mojolicious-Plugin-DataTables.git
+
+
+=head1 AUTHOR
+
+=over 4
+
+=item * Giuseppe Di Terlizzi <gdt@cpan.org>
+
+=back
+
+
+=head1 LICENSE AND COPYRIGHT
+
+This software is copyright (c) 2020-2021 by Giuseppe Di Terlizzi.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
